do

	local CustomVOReplayTime = {
		MassCritical = 300,
		EnergyDangerous = 30,
		MassDangerous = 30,
		MassDepleted = 120,
		EnergyDepleted = 120,
		UnitStartBeingBuilt = 1,
		UnitComplete = 1,
		EnemyUnitDetected = 10,
		UnitUnderAttack = 30,
		UnitHealthAt = 20,
		UnitDestroyed = 10,
		
		ProjectileIntercepted = 1,
		
		Defeated = 1,
	}
	local IsPlaying = false
	
	Marker = function(mtype, mposition)
		return {type=mtype, position=mposition}
	end

	local oldAI = AIBrain
	AIBrain = Class(oldAI) {
	
		GetFactionName = function(self)
			local factions = {'UEF', 'Aeon', 'Cybran'}
			local name = factions[self:GetFactionIndex()]
			return name
		end,
	
		InitializeEconomyState = function(self)
			oldAI.InitializeEconomyState(self)
			self.EconThread = ForkThread(self.EconomyWarningsVO, self)
		end,
		
		InitializeVO = function(self)
			VOReplayTime = table.merged(VOReplayTime, CustomVOReplayTime)
			oldAI.InitializeVO(self)
		end,
		
		PlayVOSound = function(self, sound, string)
			PlayVOSound(self, sound, string, nil)
		end,
		PlayVOSound = function(self, sound, string, marker)
			#play only for my army
			if GetFocusArmy() == self:GetArmyIndex() then
				#wait for the previous VO to finish
				while IsPlaying do
					WaitSeconds(0.5)
				end
				#extract sound and send to UI
				local cue,bank = GetCueBank(sound)
				if not Sync.UnitData.VOs then
					Sync.UnitData.VOs = {}
				end
				table.insert(Sync.UnitData.VOs, {Cue=cue, Bank=bank, Text=string, Marker=marker})
				#wait for end of delay
				local time = VOReplayTime[string]
				if not time then
					WARN('No time entry for '..string)
				end
				IsPlaying = true
				WaitSeconds(3)
				IsPlaying = false
				if time - 3 < 1 then
					time = 4
				end
				WaitSeconds(time - 3)
				self.VOTable[string] = nil
			end
		end,
	
		#####################
		#UNIT START BEING BUILT
		UnitStartBeingBuilt = function(self, sound)
			if self.VOTable and not self.VOTable['UnitStartBeingBuilt'] then
				self.VOTable['UnitStartBeingBuilt'] = ForkThread(self.PlayVOSound, self, sound, 'UnitStartBeingBuilt')
			end
		end,
		AllyUnitStartBeingBuilt = function(self, sound, position)
			if self.VOTable and not self.VOTable['UnitStartBeingBuilt'] then
				self.VOTable['UnitStartBeingBuilt'] = ForkThread(self.PlayVOSound, self, sound, 'UnitStartBeingBuilt', Marker('alertblue', position))
			end
		end,
		#####################
		#UNIT COMPLETE
		UnitComplete = function(self, sound, position)
			if self.VOTable and not self.VOTable['UnitComplete'] then
				self.VOTable['UnitComplete'] = ForkThread(self.PlayVOSound, self, sound, 'UnitComplete', Marker('alertgreen', position))
			end
		end,
		AllyUnitComplete = function(self, sound, position)
			if self.VOTable and not self.VOTable['UnitComplete'] then
				self.VOTable['UnitComplete'] = ForkThread(self.PlayVOSound, self, sound, 'UnitComplete', Marker('alertgreen', position))
			end
		end,
		#####################
		#UNIT DETECTED
		EnemyUnitDetected = function(self, sound, position)
			if self.VOTable and not self.VOTable['EnemyUnitDetected'] then
				self.VOTable['EnemyUnitDetected'] = ForkThread(self.PlayVOSound, self, sound, 'EnemyUnitDetected', Marker('alertred', position))
			end
		end,
		#####################
		#UNIT UNDER ATTACK
		UnitUnderAttack = function(self, sound, position)
			if self.VOTable and not self.VOTable['UnitUnderAttack'] then
				self.VOTable['UnitUnderAttack'] = ForkThread(self.PlayVOSound, self, sound, 'UnitUnderAttack', Marker('alertred', position))
			end
		end,
		AllyUnitUnderAttack = function(self, sound, position)
			if self.VOTable and not self.VOTable['UnitUnderAttack'] then
				self.VOTable['UnitUnderAttack'] = ForkThread(self.PlayVOSound, self, sound, 'UnitUnderAttack', Marker('alertyellow', position))
			end
		end,
		UnitHealthAt = function(self, sound)
			if self.VOTable and not self.VOTable['UnitHealthAt'] then
				self.VOTable['UnitHealthAt'] = ForkThread(self.PlayVOSound, self, sound, 'UnitHealthAt')
			end
		end,
		#####################
		#UNIT DESTROYED
		UnitDestroyed = function(self, sound, position)
			if self.VOTable and not self.VOTable['UnitDestroyed'] then
				self.VOTable['UnitDestroyed'] = ForkThread(self.PlayVOSound, self, sound, 'UnitDestroyed', Marker('alertred', position))
			end
		end,
		AllyUnitDestroyed = function(self, sound, position)
			if self.VOTable and not self.VOTable['UnitDestroyed'] then
				self.VOTable['UnitDestroyed'] = ForkThread(self.PlayVOSound, self, sound, 'UnitDestroyed', Marker('alertyellow', position))
			end
		end,
		EnemyUnitDestroyed = function(self, sound, position)
			if self.VOTable and not self.VOTable['UnitDestroyed'] then
				self.VOTable['UnitDestroyed'] = ForkThread(self.PlayVOSound, self, sound, 'UnitDestroyed', Marker('alertgreen', position))
			end
		end,
		################
		#STRATEGIC WEAPONARY
		NukeArmed = function(self, sound, position)
			if self.VOTable and not self.VOTable['NukeArmed'] then
				self.VOTable['NukeArmed'] = ForkThread(self.PlayVOSound, self, sound, 'NukeArmed', Marker('alertgreen', position))
			end
		end,
		AllyNukeArmed = function(self, sound, position)
			if self.VOTable and not self.VOTable['NukeArmed'] then
				self.VOTable['NukeArmed'] = ForkThread(self.PlayVOSound, self, sound, 'NukeArmed', Marker('alertblue', position))
			end
		end,
		EnemyNukeArmed = function(self, sound, position)
			if self.VOTable and not self.VOTable['NukeArmed'] then
				self.VOTable['NukeArmed'] = ForkThread(self.PlayVOSound, self, sound, 'NukeArmed', Marker('alertred', position))
			end
		end,
		ProjectileIntercepted = function(self, sound)
			if self.VOTable and not self.VOTable['ProjectileIntercepted'] then
				self.VOTable['ProjectileIntercepted'] = ForkThread(self.PlayVOSound, self, sound, 'ProjectileIntercepted')
			end
		end,
		EnemyProjectileIntercepted = function(self, sound)
			if self.VOTable and not self.VOTable['ProjectileIntercepted'] then
				self.VOTable['ProjectileIntercepted'] = ForkThread(self.PlayVOSound, self, sound, 'ProjectileIntercepted')
			end
		end,
		################
		
		OnDefeat = function(self)
			KillThread(self.EconThread)
			if self.VOTable and not self.VOTable['Defeated'] then
				local FactionName = self:GetFactionName()
				local sound = Sound {
					Bank = 'COMPUTER_'..FactionName..'_VO',
					Cue = FactionName..'Computer_Failed_01422',}
				self.VOTable['Defeated'] = ForkThread(self.PlayVOSound, self, sound, 'Defeated')
			end
			oldAI.OnDefeat(self)
		end,
	
		EconomyWarningsVO = function(self)
			WaitSeconds(10)
			local AIu = import('/lua/AI/aiutilities.lua')
			while true do
				local Econ = AIu.AIGetEconomyNumbers(self)
				local FactionName = self:GetFactionName()
				if Econ.EnergyStorageRatio == 0 then
					if self.VOTable and not self.VOTable['EnergyDepleted'] then
						local sound = Sound {
							Bank = 'COMPUTER_'..FactionName..'_VO',
							Cue = FactionName..'Computer_Resources_01308',}
						self.VOTable['EnergyDepleted'] = ForkThread(self.PlayVOSound, self, sound, 'EnergyDepleted')
					end
				elseif Econ.EnergyStorageRatio < 0.3 and Econ.EnergyTrend < 0 then
					if self.VOTable and not self.VOTable['EnergyDangerous'] then
						local sound = Sound {
							Bank = 'COMPUTER_'..FactionName..'_VO',
							Cue = FactionName..'Computer_Resources_01304',}
						self.VOTable['EnergyDangerous'] = ForkThread(self.PlayVOSound, self, sound, 'EnergyDangerous')
					end
				end
				if Econ.MassStorageRatio == 0 then
					if self.VOTable and not self.VOTable['MassDepleted'] then
						local sound = Sound {
							Bank = 'COMPUTER_'..FactionName..'_VO',
							Cue = FactionName..'Computer_Resources_01307',}
						self.VOTable['MassDepleted'] = ForkThread(self.PlayVOSound, self, sound, 'MassDepleted')
					end
				elseif Econ.MassStorageRatio < 0.1 and Econ.MassTrend < 0 then
					if self.VOTable and not self.VOTable['MassDangerous'] then
						local sound = Sound {
							Bank = 'COMPUTER_'..FactionName..'_VO',
							Cue = FactionName..'Computer_Resources_01303',}
						self.VOTable['MassDangerous'] = ForkThread(self.PlayVOSound, self, sound, 'MassDangerous')
					end
				elseif Econ.MassStorageRatio > 0.8 and Econ.MassTrend > 0 then
					if self.VOTable and not self.VOTable['MassCritical'] then
						local sound = Sound {
							Bank = 'COMPUTER_'..FactionName..'_VO',
							Cue = FactionName..'Computer_Resources_01305',}
						self.VOTable['MassCritical'] = ForkThread(self.PlayVOSound, self, sound, 'MassCritical')
					end
				end
				WaitSeconds(1)
			end
		end,
	}
end